# This file was automatically generated by SWIG (https://www.swig.org).
# Version 4.4.1
#
# Do not make changes to this file unless you know what you are doing - modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
# Import the low-level C/C++ module
if getattr(globals().get("__spec__"), "parent", None) or __package__ or "." in __name__:
    from . import _icalmapi
else:
    import _icalmapi

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "this":
            set(self, name, value)
        elif name == "thisown":
            self.this.own(value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


VEVENT = _icalmapi.VEVENT
VTODO = _icalmapi.VTODO
VJOURNAL = _icalmapi.VJOURNAL
IC2M_NO_RECIPIENTS = _icalmapi.IC2M_NO_RECIPIENTS
IC2M_APPEND_ONLY = _icalmapi.IC2M_APPEND_ONLY
IC2M_NO_ORGANIZER = _icalmapi.IC2M_NO_ORGANIZER
IC2M_NO_BODY = _icalmapi.IC2M_NO_BODY
class ICalToMapi(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _icalmapi.delete_ICalToMapi

    def ParseICal2(self, ical, charset, server_tz, arg5, flags):
        return _icalmapi.ICalToMapi_ParseICal2(self, ical, charset, server_tz, arg5, flags)

    def ParseICal(self, ical, charset, server_tz, mu, flags):
        return _icalmapi.ICalToMapi_ParseICal(self, ical, charset, server_tz, mu, flags)

    def GetItemCount(self):
        return _icalmapi.ICalToMapi_GetItemCount(self)

    def GetItemInfo(self, ulPosition):
        return _icalmapi.ICalToMapi_GetItemInfo(self, ulPosition)

    def GetItem(self, ulPosition, ulFlags, lpMessage):
        return _icalmapi.ICalToMapi_GetItem(self, ulPosition, ulFlags, lpMessage)

    def GetFreeBusyInfo(self, users):
        return _icalmapi.ICalToMapi_GetFreeBusyInfo(self, users)

    def GetNumInvalidProperties(self):
        return _icalmapi.ICalToMapi_GetNumInvalidProperties(self)

    def GetNumInvalidComponents(self):
        return _icalmapi.ICalToMapi_GetNumInvalidComponents(self)

# Register ICalToMapi in _icalmapi:
_icalmapi.ICalToMapi_swigregister(ICalToMapi)

def CreateICalToMapi(propobj, arg2, no_recipients):
    return _icalmapi.CreateICalToMapi(propobj, arg2, no_recipients)
M2IC_CENSOR_PRIVATE = _icalmapi.M2IC_CENSOR_PRIVATE
M2IC_NO_VTIMEZONE = _icalmapi.M2IC_NO_VTIMEZONE
class MapiToICal(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _icalmapi.delete_MapiToICal

    def AddMessage(self, lpMessage, strSrvTZ, ulFlags):
        return _icalmapi.MapiToICal_AddMessage(self, lpMessage, strSrvTZ, ulFlags)

    def AddBlocks(self, lpsFBblk, ulBlocks, tStart, tEnd, strOrganiser, strUser, strUID):
        return _icalmapi.MapiToICal_AddBlocks(self, lpsFBblk, ulBlocks, tStart, tEnd, strOrganiser, strUser, strUID)

    def Finalize(self, ulFlags):
        return _icalmapi.MapiToICal_Finalize(self, ulFlags)

    def ResetObject(self):
        return _icalmapi.MapiToICal_ResetObject(self)

# Register MapiToICal in _icalmapi:
_icalmapi.MapiToICal_swigregister(MapiToICal)

def CreateMapiToICal(arg1, charset):
    return _icalmapi.CreateMapiToICal(arg1, charset)
class mapitovcf(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _icalmapi.delete_mapitovcf

    def add_message(self, arg2):
        return _icalmapi.mapitovcf_add_message(self, arg2)

    def finalize(self):
        return _icalmapi.mapitovcf_finalize(self)

# Register mapitovcf in _icalmapi:
_icalmapi.mapitovcf_swigregister(mapitovcf)

def create_mapitovcf():
    return _icalmapi.create_mapitovcf()
class vcftomapi(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _icalmapi.delete_vcftomapi

    def parse_vcf(self, ical):
        return _icalmapi.vcftomapi_parse_vcf(self, ical)

    def get_item_count(self):
        return _icalmapi.vcftomapi_get_item_count(self)

    def get_item(self, arg2, section=0):
        return _icalmapi.vcftomapi_get_item(self, arg2, section)

# Register vcftomapi in _icalmapi:
_icalmapi.vcftomapi_swigregister(vcftomapi)

def create_vcftomapi(arg1):
    return _icalmapi.create_vcftomapi(arg1)

