#define PROJECT_VERSION                 "11.0.2"
#define PROJECT_VERSION_MAJOR           11
#define PROJECT_VERSION_MINOR           0
#define PROJECT_VERSION_MICRO           2
#define PROJECT_VERSION_REVISION        0UL
#define PROJECT_VERSION_COMMIFIED       "11,0,2"
