/* common/include/kopano/buildconfig.h.  Generated from buildconfig.h.in by configure.  */
/*
 * This file should not be included directly, consider using
 * <kopano/zcdefs.h> instead.
 */
#define KC_USES_CXX17 1
#define KC_USES_TIMEGM 1
