# This file was automatically generated by SWIG (https://www.swig.org).
# Version 4.4.1
#
# Do not make changes to this file unless you know what you are doing - modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
# Import the low-level C/C++ module
if getattr(globals().get("__spec__"), "parent", None) or __package__ or "." in __name__:
    from . import _libfreebusy
else:
    import _libfreebusy

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "this":
            set(self, name, value)
        elif name == "thisown":
            self.this.own(value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


fbFree = _libfreebusy.fbFree
fbTentative = _libfreebusy.fbTentative
fbBusy = _libfreebusy.fbBusy
fbOutOfOffice = _libfreebusy.fbOutOfOffice
fbKopanoAllBusy = _libfreebusy.fbKopanoAllBusy
class IFreeBusyUpdate(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    def PublishFreeBusy(self, arg2):
        return _libfreebusy.IFreeBusyUpdate_PublishFreeBusy(self, arg2)

    def ResetPublishedFreeBusy(self):
        return _libfreebusy.IFreeBusyUpdate_ResetPublishedFreeBusy(self)

    def SaveChanges(self, start, end):
        return _libfreebusy.IFreeBusyUpdate_SaveChanges(self, start, end)
    __swig_destroy__ = _libfreebusy.delete_IFreeBusyUpdate

# Register IFreeBusyUpdate in _libfreebusy:
_libfreebusy.IFreeBusyUpdate_swigregister(IFreeBusyUpdate)
class IEnumFBBlock(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    def Next(self, celt):
        return _libfreebusy.IEnumFBBlock_Next(self, celt)

    def Skip(self, celt):
        return _libfreebusy.IEnumFBBlock_Skip(self, celt)

    def Reset(self):
        return _libfreebusy.IEnumFBBlock_Reset(self)

    def Clone(self):
        return _libfreebusy.IEnumFBBlock_Clone(self)

    def Restrict(self, start, end):
        return _libfreebusy.IEnumFBBlock_Restrict(self, start, end)
    __swig_destroy__ = _libfreebusy.delete_IEnumFBBlock

# Register IEnumFBBlock in _libfreebusy:
_libfreebusy.IEnumFBBlock_swigregister(IEnumFBBlock)
class IFreeBusyData(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    def EnumBlocks(self, start, end):
        return _libfreebusy.IFreeBusyData_EnumBlocks(self, start, end)

    def GetDelegateInfo(self, arg2):
        return _libfreebusy.IFreeBusyData_GetDelegateInfo(self, arg2)

    def SetFBRange(self, rtmStart, rtmEnd):
        return _libfreebusy.IFreeBusyData_SetFBRange(self, rtmStart, rtmEnd)

    def GetFBPublishRange(self):
        return _libfreebusy.IFreeBusyData_GetFBPublishRange(self)
    __swig_destroy__ = _libfreebusy.delete_IFreeBusyData

# Register IFreeBusyData in _libfreebusy:
_libfreebusy.IFreeBusyData_swigregister(IFreeBusyData)
class IFreeBusySupport(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def Open(self, lpMAPISession, lpMsgStore, bStore):
        return _libfreebusy.IFreeBusySupport_Open(self, lpMAPISession, lpMsgStore, bStore)

    def Close(self):
        return _libfreebusy.IFreeBusySupport_Close(self)

    def LoadFreeBusyData(self, cMax, phrStatus):
        return _libfreebusy.IFreeBusySupport_LoadFreeBusyData(self, cMax, phrStatus)

    def LoadFreeBusyUpdate(self, cUsers, lpData4):
        return _libfreebusy.IFreeBusySupport_LoadFreeBusyUpdate(self, cUsers, lpData4)

    def CommitChanges(self):
        return _libfreebusy.IFreeBusySupport_CommitChanges(self)

    def GetDelegateInfo(self, arg2, arg3):
        return _libfreebusy.IFreeBusySupport_GetDelegateInfo(self, arg2, arg3)

    def SetDelegateInfo(self, arg2):
        return _libfreebusy.IFreeBusySupport_SetDelegateInfo(self, arg2)

    def AdviseFreeBusy(self, arg2):
        return _libfreebusy.IFreeBusySupport_AdviseFreeBusy(self, arg2)

    def GetFBDetailSupport(self, arg2, arg3):
        return _libfreebusy.IFreeBusySupport_GetFBDetailSupport(self, arg2, arg3)

    def HrHandleServerSched(self, arg2):
        return _libfreebusy.IFreeBusySupport_HrHandleServerSched(self, arg2)

    def HrHandleServerSchedAccess(self):
        return _libfreebusy.IFreeBusySupport_HrHandleServerSchedAccess(self)

    def FShowServerSched(self, arg2):
        return _libfreebusy.IFreeBusySupport_FShowServerSched(self, arg2)

    def HrDeleteServerSched(self):
        return _libfreebusy.IFreeBusySupport_HrDeleteServerSched(self)

    def GetFReadOnly(self, arg2):
        return _libfreebusy.IFreeBusySupport_GetFReadOnly(self, arg2)

    def SetLocalFB(self, arg2):
        return _libfreebusy.IFreeBusySupport_SetLocalFB(self, arg2)

    def PrepareForSync(self):
        return _libfreebusy.IFreeBusySupport_PrepareForSync(self)

    def GetFBPublishMonthRange(self, arg2):
        return _libfreebusy.IFreeBusySupport_GetFBPublishMonthRange(self, arg2)

    def PublishRangeChanged(self):
        return _libfreebusy.IFreeBusySupport_PublishRangeChanged(self)

    def CleanTombstone(self):
        return _libfreebusy.IFreeBusySupport_CleanTombstone(self)

    def GetDelegateInfoEx(self, arg2, status, start, end):
        return _libfreebusy.IFreeBusySupport_GetDelegateInfoEx(self, arg2, status, start, end)

    def PushDelegateInfoToWorkspace(self):
        return _libfreebusy.IFreeBusySupport_PushDelegateInfoToWorkspace(self)

    def __init__(self):
        _libfreebusy.IFreeBusySupport_swiginit(self, _libfreebusy.new_IFreeBusySupport())
    __swig_destroy__ = _libfreebusy.delete_IFreeBusySupport

# Register IFreeBusySupport in _libfreebusy:
_libfreebusy.IFreeBusySupport_swigregister(IFreeBusySupport)

