# This file was automatically generated by SWIG (https://www.swig.org).
# Version 4.4.1
#
# Do not make changes to this file unless you know what you are doing - modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
# Import the low-level C/C++ module
if getattr(globals().get("__spec__"), "parent", None) or __package__ or "." in __name__:
    from . import _inetmapi
else:
    import _inetmapi

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "this":
            set(self, name, value)
        elif name == "thisown":
            self.this.own(value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class sending_options(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    alternate_boundary = property(_inetmapi.sending_options_alternate_boundary_get, _inetmapi.sending_options_alternate_boundary_set)
    no_recipients_workaround = property(_inetmapi.sending_options_no_recipients_workaround_get, _inetmapi.sending_options_no_recipients_workaround_set)
    msg_in_msg = property(_inetmapi.sending_options_msg_in_msg_get, _inetmapi.sending_options_msg_in_msg_set)
    headers_only = property(_inetmapi.sending_options_headers_only_get, _inetmapi.sending_options_headers_only_set)
    add_received_date = property(_inetmapi.sending_options_add_received_date_get, _inetmapi.sending_options_add_received_date_set)
    use_tnef = property(_inetmapi.sending_options_use_tnef_get, _inetmapi.sending_options_use_tnef_set)
    charset_upgrade = property(_inetmapi.sending_options_charset_upgrade_get, _inetmapi.sending_options_charset_upgrade_set)
    allow_send_to_everyone = property(_inetmapi.sending_options_allow_send_to_everyone_get, _inetmapi.sending_options_allow_send_to_everyone_set)
    enable_dsn = property(_inetmapi.sending_options_enable_dsn_get, _inetmapi.sending_options_enable_dsn_set)

    def __init__(self):
        _inetmapi.sending_options_swiginit(self, _inetmapi.new_sending_options())
    __swig_destroy__ = _inetmapi.delete_sending_options

# Register sending_options in _inetmapi:
_inetmapi.sending_options_swigregister(sending_options)
class delivery_options(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    use_received_date = property(_inetmapi.delivery_options_use_received_date_get, _inetmapi.delivery_options_use_received_date_set)
    mark_as_read = property(_inetmapi.delivery_options_mark_as_read_get, _inetmapi.delivery_options_mark_as_read_set)
    add_imap_data = property(_inetmapi.delivery_options_add_imap_data_get, _inetmapi.delivery_options_add_imap_data_set)
    parse_smime_signed = property(_inetmapi.delivery_options_parse_smime_signed_get, _inetmapi.delivery_options_parse_smime_signed_set)
    ascii_upgrade = property(_inetmapi.delivery_options_ascii_upgrade_get, _inetmapi.delivery_options_ascii_upgrade_set)
    header_strict_rfc = property(_inetmapi.delivery_options_header_strict_rfc_get, _inetmapi.delivery_options_header_strict_rfc_set)
    charset_strict_rfc = property(_inetmapi.delivery_options_charset_strict_rfc_get, _inetmapi.delivery_options_charset_strict_rfc_set)

    def __init__(self):
        _inetmapi.delivery_options_swiginit(self, _inetmapi.new_delivery_options())
    __swig_destroy__ = _inetmapi.delete_delivery_options

# Register delivery_options in _inetmapi:
_inetmapi.delivery_options_swigregister(delivery_options)

def IMToMAPI(lpSession, lpMsgStore, lpAddrBook, lpMessage, input, dopt):
    return _inetmapi.IMToMAPI(lpSession, lpMsgStore, lpAddrBook, lpMessage, input, dopt)

def IMToINet(lpSession, lpAddrBook, lpMessage, sopt):
    return _inetmapi.IMToINet(lpSession, lpAddrBook, lpMessage, sopt)

def createIMAPProperties(input):
    return _inetmapi.createIMAPProperties(input)

